

# Module dqe_idx #
* [Description](#description)
* [Data Types](#types)
* [Function Index](#index)
* [Function Details](#functions)

Call back module and wrapper for the Dalmatiner Query Engine
Indexer.

Copyright (c) (C) 2016, Project-FiFo UG

__This module defines the `dqe_idx` behaviour.__<br /> Required callback functions: `init/0`, `lookup/1`, `lookup/2`, `lookup_tags/1`, `collections/0`, `metrics/1`, `namespaces/2`, `tags/3`, `values/4`, `expand/2`, `add/4`, `add/5`, `update/5`, `delete/4`, `delete/5`.

<a name="description"></a>

## Description ##
This module should be used in place of calling
different indexer backends.
<a name="types"></a>

## Data Types ##




### <a name="type-bucket">bucket()</a> ###


<pre><code>
bucket() = binary()
</code></pre>




### <a name="type-collection">collection()</a> ###


<pre><code>
collection() = binary()
</code></pre>




### <a name="type-glob_metric">glob_metric()</a> ###


<pre><code>
glob_metric() = [binary() | '*']
</code></pre>




### <a name="type-group_by_field">group_by_field()</a> ###


<pre><code>
group_by_field() = binary()
</code></pre>




### <a name="type-key">key()</a> ###


<pre><code>
key() = binary()
</code></pre>




### <a name="type-lqry">lqry()</a> ###


<pre><code>
lqry() = {in, <a href="#type-collection">collection()</a>, [<a href="#type-metric">metric()</a>]} | {in, <a href="#type-collection">collection()</a>, [<a href="#type-metric">metric()</a>], <a href="#type-where">where()</a>}
</code></pre>




### <a name="type-metric">metric()</a> ###


<pre><code>
metric() = binary()
</code></pre>




### <a name="type-namespace">namespace()</a> ###


<pre><code>
namespace() = binary()
</code></pre>




### <a name="type-tag">tag()</a> ###


<pre><code>
tag() = {tag, Namespace::<a href="#type-namespace">namespace()</a>, TagName::<a href="#type-tag_name">tag_name()</a>}
</code></pre>




### <a name="type-tag_name">tag_name()</a> ###


<pre><code>
tag_name() = binary()
</code></pre>




### <a name="type-tag_value">tag_value()</a> ###


<pre><code>
tag_value() = binary()
</code></pre>




### <a name="type-tags">tags()</a> ###


<pre><code>
tags() = [{Namespace::<a href="#type-namespace">namespace()</a>, TagName::<a href="#type-tag_name">tag_name()</a>, Value::<a href="#type-tag_value">tag_value()</a>}]
</code></pre>




### <a name="type-where">where()</a> ###


<pre><code>
where() = {'=', <a href="#type-tag">tag()</a>, <a href="#type-tag_value">tag_value()</a>} | {'and', <a href="#type-where">where()</a>, <a href="#type-where">where()</a>} | {'or', <a href="#type-where">where()</a>, <a href="#type-where">where()</a>}
</code></pre>

<a name="index"></a>

## Function Index ##


<table width="100%" border="1" cellspacing="0" cellpadding="2" summary="function index"><tr><td valign="top"><a href="#add-4">add/4</a></td><td>
Links a collection/metric to a bucket and key.</td></tr><tr><td valign="top"><a href="#add-5">add/5</a></td><td>
Adds one or more metrics tag pairs to a metric.</td></tr><tr><td valign="top"><a href="#collections-0">collections/0</a></td><td>
Lists all collections.</td></tr><tr><td valign="top"><a href="#delete-4">delete/4</a></td><td>
Deletes a Collection/Metric pair and all it's tags.</td></tr><tr><td valign="top"><a href="#delete-5">delete/5</a></td><td>
Deletes one or more Tag pairs from a Metric.</td></tr><tr><td valign="top"><a href="#expand-2">expand/2</a></td><td>
Expands a glob into all matching metrics for a given bucket.</td></tr><tr><td valign="top"><a href="#init-0">init/0</a></td><td>
Initializes the Dalmatiner Query Engine indexer, this will hand
down to whatever indexing backend is used.</td></tr><tr><td valign="top"><a href="#lookup-1">lookup/1</a></td><td>
Takes a lookup query and reutrns a list of all metric/bucket
paris that metch the lookup criteria.</td></tr><tr><td valign="top"><a href="#lookup-2">lookup/2</a></td><td>
Takes a lookup query and reutrns a list of all metric/bucket
paris that metch the lookup criteria with the values for the
provided group by fields (tags).</td></tr><tr><td valign="top"><a href="#lookup_tags-1">lookup_tags/1</a></td><td>
Find all possible namespace, tag, value pairs for a query.</td></tr><tr><td valign="top"><a href="#metrics-1">metrics/1</a></td><td>
Lists all metrics in a collections.</td></tr><tr><td valign="top"><a href="#namespaces-2">namespaces/2</a></td><td>
Lists all namespaces for a metrics.</td></tr><tr><td valign="top"><a href="#tags-3">tags/3</a></td><td>
Lists all tags for a namespaces metrics.</td></tr><tr><td valign="top"><a href="#update-5">update/5</a></td><td>
Updates values of a metric, this behaves equivalent to add/5 if
the metric was not yet known to the index store, if however it was
known it MUST add new tags and MUST update existing tags.</td></tr><tr><td valign="top"><a href="#values-4">values/4</a></td><td>
Lists all the possible values for a tag.</td></tr></table>


<a name="functions"></a>

## Function Details ##

<a name="add-4"></a>

### add/4 ###

<pre><code>
add(Collection::<a href="#type-collection">collection()</a>, Metric::<a href="#type-metric">metric()</a>, Bucket::<a href="#type-bucket">bucket()</a>, Key::<a href="#type-key">key()</a>) -&gt; {ok, MetricIdx::term()} | ok | {error, Error::term()}
</code></pre>
<br />

Links a collection/metric to a bucket and key. Returns whatever
identifyer the colleciton/metric has if any. This MAY either return
{ok, ID} or optinally ok, if ok is returned it MUST only be done
if the metric was already present in the index store. Returning
{ok, ID} is ALWAYS acceptable. A consumer of this API MAY assume
that if ok is returned the related tags are already in the store
as well.

<a name="add-5"></a>

### add/5 ###

<pre><code>
add(Collection::<a href="#type-collection">collection()</a>, Metric::<a href="#type-metric">metric()</a>, Bucket::<a href="#type-bucket">bucket()</a>, Key::<a href="#type-key">key()</a>, Tags::[{<a href="#type-namespace">namespace()</a>, <a href="#type-tag_name">tag_name()</a>, <a href="#type-tag_value">tag_value()</a>}]) -&gt; {ok, MetricIdx::term()} | ok | {error, Error::term()}
</code></pre>
<br />

Adds one or more metrics tag pairs to a metric. This
function MUST not change existing tags, or add tags to an existing
metric IF add/4 returned ok. It MAY add additional tags if add/4
returned {ok, ID} despite the metric being present.

<a name="collections-0"></a>

### collections/0 ###

<pre><code>
collections() -&gt; {ok, [<a href="#type-collection">collection()</a>]} | {error, Error::term()}
</code></pre>
<br />

Lists all collections.

<a name="delete-4"></a>

### delete/4 ###

<pre><code>
delete(Collection::<a href="#type-collection">collection()</a>, Metric::<a href="#type-metric">metric()</a>, Bucket::<a href="#type-bucket">bucket()</a>, Key::<a href="#type-key">key()</a>) -&gt; ok | {error, Error::term()}
</code></pre>
<br />

Deletes a Collection/Metric pair and all it's tags.

<a name="delete-5"></a>

### delete/5 ###

<pre><code>
delete(Collection::<a href="#type-collection">collection()</a>, Metric::<a href="#type-metric">metric()</a>, Bucket::<a href="#type-bucket">bucket()</a>, Key::<a href="#type-key">key()</a>, Tags::[{<a href="#type-namespace">namespace()</a>, <a href="#type-tag_name">tag_name()</a>, <a href="#type-tag_value">tag_value()</a>}]) -&gt; ok | {error, Error::term()}
</code></pre>
<br />

Deletes one or more Tag pairs from a Metric. This funciton can
call delete/6 multiple times or use a more optimized method. This
MUST only be used with metric2.0 like metadata tags that do not
change metric identity!

<a name="expand-2"></a>

### expand/2 ###

<pre><code>
expand(B::<a href="#type-bucket">bucket()</a>, Gs::[<a href="#type-glob_metric">glob_metric()</a>]) -&gt; {ok, {<a href="#type-bucket">bucket()</a>, [<a href="#type-metric">metric()</a>]}} | {error, Error::term()}
</code></pre>
<br />

Expands a glob into all matching metrics for a given bucket.
WARNING: This might go away!

<a name="init-0"></a>

### init/0 ###

<pre><code>
init() -&gt; ok | {error, Error::term()}
</code></pre>
<br />

Initializes the Dalmatiner Query Engine indexer, this will hand
down to whatever indexing backend is used.

<a name="lookup-1"></a>

### lookup/1 ###

<pre><code>
lookup(Query::<a href="#type-lqry">lqry()</a>) -&gt; {ok, [{<a href="#type-bucket">bucket()</a>, <a href="#type-key">key()</a>}]} | {error, Error::term()}
</code></pre>
<br />

Takes a lookup query and reutrns a list of all metric/bucket
paris that metch the lookup criteria.

<a name="lookup-2"></a>

### lookup/2 ###

<pre><code>
lookup(Query::<a href="#type-lqry">lqry()</a>, GroupBy::[<a href="#type-group_by_field">group_by_field()</a>]) -&gt; {ok, [{<a href="#type-bucket">bucket()</a>, <a href="#type-key">key()</a>, [<a href="#type-group_by_field">group_by_field()</a>]}]} | {error, Error::term()}
</code></pre>
<br />

Takes a lookup query and reutrns a list of all metric/bucket
paris that metch the lookup criteria with the values for the
provided group by fields (tags).

<a name="lookup_tags-1"></a>

### lookup_tags/1 ###

<pre><code>
lookup_tags(Query::<a href="#type-lqry">lqry()</a>) -&gt; {ok, <a href="#type-tags">tags()</a>} | {error, Error::term()}
</code></pre>
<br />

Find all possible namespace, tag, value pairs for a query.

<a name="metrics-1"></a>

### metrics/1 ###

<pre><code>
metrics(Collection::<a href="#type-collection">collection()</a>) -&gt; {ok, [<a href="#type-metric">metric()</a>]} | {error, Error::term()}
</code></pre>
<br />

Lists all metrics in a collections.

<a name="namespaces-2"></a>

### namespaces/2 ###

<pre><code>
namespaces(Collection::<a href="#type-collection">collection()</a>, Metric::<a href="#type-metric">metric()</a>) -&gt; {ok, [<a href="#type-namespace">namespace()</a>]} | {error, Error::term()}
</code></pre>
<br />

Lists all namespaces for a metrics.

<a name="tags-3"></a>

### tags/3 ###

<pre><code>
tags(Collection::<a href="#type-collection">collection()</a>, Metric::<a href="#type-metric">metric()</a>, Namesplace::<a href="#type-namespace">namespace()</a>) -&gt; {ok, [<a href="#type-tag_name">tag_name()</a>]} | {error, Error::term()}
</code></pre>
<br />

Lists all tags for a namespaces metrics.

<a name="update-5"></a>

### update/5 ###

<pre><code>
update(Collection::<a href="#type-collection">collection()</a>, Metric::<a href="#type-metric">metric()</a>, Bucket::<a href="#type-bucket">bucket()</a>, Key::<a href="#type-key">key()</a>, Tags::[{<a href="#type-namespace">namespace()</a>, <a href="#type-tag_name">tag_name()</a>, <a href="#type-tag_value">tag_value()</a>}]) -&gt; {ok, MetricIdx::term()} | ok | {error, Error::term()}
</code></pre>
<br />

Updates values of a metric, this behaves equivalent to add/5 if
the metric was not yet known to the index store, if however it was
known it MUST add new tags and MUST update existing tags. This is
meant to be used with metadata tags as described in the metrics2.0
specification.

<a name="values-4"></a>

### values/4 ###

<pre><code>
values(Collection::<a href="#type-collection">collection()</a>, Metric::<a href="#type-metric">metric()</a>, Namesplace::<a href="#type-namespace">namespace()</a>, Tag::<a href="#type-tag_name">tag_name()</a>) -&gt; {ok, [<a href="#type-tag_value">tag_value()</a>]} | {error, Error::term()}
</code></pre>
<br />

Lists all the possible values for a tag

